==> rovm-0.0.10a.tar.bz2: rovm/src/ticket.h <==
/*  -
   Copyright (C) 2006 Weongyo Jeong (weongyo@gmail.com)

This file is part of ROVM.

ROVM is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free
Software Foundation; either version 2, or (at your option) any later
version.

ROVM is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with GCC; see the file COPYING.  If not, write to the Free
Software Foundation, 59 Temple Place - Suite 330, Boston, MA
02111-1307, USA.  */

#ifndef _ROVM_TICKET_H
#define _ROVM_TICKET_H

#include "core.h"
#include "class.h"

/**
   ⺻   slot  .
 */
#define DEFAULT_STACK_SLOT              256

#define TICKET_POOL(NODE)               ((NODE)->p)
#define TICKET_MUTEX(NODE)              ((NODE)->mutex)
#define TICKET_TIME(NODE)		((NODE)->tmade)

#define TICKET_SP(NODE)                 ((NODE)->sp)
#define TICKET_MAX_SP(NODE)             ((NODE)->maxsp)
#define TICKET_MIN_SP(NODE)             ((NODE)->minsp)
#define TICKET_LOCAL(NODE)              ((NODE)->local)
#define TICKET_MAXLOCAL(NODE)           ((NODE)->maxlocal)
#define TICKET_RET(NODE)                (&((NODE)->ret))

#define TICKET_STACK(NODE)              ((NODE)->stack)
#define TICKET_STACK_IDX(NODE, IDX)     ((NODE)->stack[(IDX)])
#define TICKET_MAX_STACK_SLOT(NODE)     ((NODE)->max_stack_slot)

/**
    request   , Ǵ üν, ڰ   
   µ  , ̸ ̿Ͽ  û     
   ֵ Ѵ.

    ü  ߿ üν  request     µ
    ִ.
 */
struct ticket
{
  /** Ticket   ޸ pool.  */
  rc_pool_t *p;

  /**
     Ticket   mutex.  ̰ GC thread  VM  ۵ İ 
       ϱ  Ǵ mutex ̴.  ڼ ϸ Ʒ .
     
      GC thread  , Ticket Tree  walking ϸ鼭 ̷ Ǵµ,
     Ʒ   Ȳ ߻  ֽϴ.

       * gc_alloc () Լ  Ҵ   GC  ̷ ,
         ( , multi thread ̱   ߻  ֽϴ.)
          Ҵ ޸𸮰 Stack Slot  ö    ֽϴ.
         ̷ ؼ Ҵ ޸𸮰 ״ GC óǴ һ簡 ߻ 
         ֽϴ.

      ߻  ִ  Ȳ   mutex Դϴ.  ذ 
       ϴ.

       1.  Ticket ü mutex   մϴ.
       2. ׸ gc_alloc () Լ   ִ opcode 鿡 Ͽ locking
          Ŀ մϴ.
       3. GC  ̷  Ticket ü locking Ŀ  
          մϴ.
  */
  rc_thread_mutex_t *mutex;

  /**
     Ticket   ð  ִ.  ̰ ߿  ð 
     ticket  , ڵ Ҹ ؼ ȴ.
  */
  rc_time_t tmade;

  /** Stack Pointer.  x86  stack pointer ̶ ϴ.  */
  rovm_stack_t *sp;
  /** Stack Pointer  ִ/ּҷ Ͽ  ̴.  ̰ Stack 
      Ѿ  ˻ϱ  ƾ̴.  */
  rovm_stack_t *maxsp;
  rovm_stack_t *minsp;

  /** Local Variable Pointer.  ̰ ڹ  ӽſ   
      Ʒ  `stack' 迭 ϳ slot  Ű ȴ.  ̿  ڼ
       `Stack  Local  '  ϱ ٶ.

      @note  `Stack  Local  '   ʴ´.  */
  rovm_stack_t *local;
  /** Local  ִ slot      , iload  
      opcode  ߸ local     ħϴ Ǵϴµ,
      ȴ.  */
  rovm_stack_t *maxlocal;

  /** Ticket  ϴ  slot .   迭Դϴ. */
  rovm_stack_t *stack;
  /**   `stack'  Ҵ  slot  .  */
  int max_stack_slot;

  /**  ticket operation  óǸ鼭 ȯϰ Ǵ return  Ǵ
      ġ.  */
  rovm_stack_t ret;
};

extern char *create_ticketid (request_rec *);
extern int gen_unique_ticketid (char *);
extern int register_ticketid (request_rec *, char *);
extern struct ticket *lookup_ticket (request_rec *, char *);
extern int ticket_pre_config ( struct ticket *);
extern int ticket_beforecall (struct ticket *, RvMethod *, u2, u2 *);
extern void destroy_ticket (struct ticket *);
extern int destroy_tickets (request_rec *, char *);

#endif

==> rovm-0.0.20a.tar.bz2: rovm/src/ticket.h <==
/*  -
   Copyright (C) 2006 Weongyo Jeong (weongyo@gmail.com)

This file is part of ROVM.

ROVM is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free
Software Foundation; either version 2, or (at your option) any later
version.

ROVM is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with GCC; see the file COPYING.  If not, write to the Free
Software Foundation, 59 Temple Place - Suite 330, Boston, MA
02111-1307, USA.  */

#ifndef _ROVM_TICKET_H
#define _ROVM_TICKET_H

#include "core.h"
#include "class.h"

/**
   ⺻   slot  .
 */
#define DEFAULT_STACK_SLOT              256

#define TICKET_TIME(NODE)		((NODE)->tmade)

#define TICKET_SP(NODE)                 ((NODE)->sp)
#define TICKET_MAX_SP(NODE)             ((NODE)->maxsp)
#define TICKET_MIN_SP(NODE)             ((NODE)->minsp)
#define TICKET_LOCAL(NODE)              ((NODE)->local)
#define TICKET_MAXLOCAL(NODE)           ((NODE)->maxlocal)
#define TICKET_RET(NODE)                (&((NODE)->ret))
#define TICKET_REAL_DELAYED_SLOT(NODE)  ((NODE)->real_delayed_slot)
#define TICKET_REAL_DELAYED_SLOT_IDX(NODE, IDX)  ((NODE)->real_delayed_slot[(IDX)])
#define TICKET_DELAYED_SLOT_MAX(NODE)   ((NODE)->delayed_slot_max)
#define TICKET_DELAYED_SLOT_NELM(NODE)  ((NODE)->delayed_slot_nelm)
#define DEFAULT_DELAYED_SLOT            64

#define TICKET_STACK(NODE)              ((NODE)->stack)
#define TICKET_STACK_IDX(NODE, IDX)     ((NODE)->stack[(IDX)])
#define TICKET_MAX_STACK_SLOT(NODE)     ((NODE)->max_stack_slot)

/**
    request   , Ǵ üν, ڰ   
   µ  , ̸ ̿Ͽ  û     
   ֵ Ѵ.

    ü  ߿ üν  request     µ
    ִ.
 */
struct ticket
{
  /**
     Ticket   ð  ִ.  ̰ ߿  ð 
     ticket  , ڵ Ҹ ؼ ȴ.
  */
  rc_time_t tmade;

  /** Stack Pointer.  x86  stack pointer ̶ ϴ.  */
  rovm_stack_t *sp;
  /** Stack Pointer  ִ/ּҷ Ͽ  ̴.  ̰ Stack 
      Ѿ  ˻ϱ  ƾ̴.  */
  rovm_stack_t *maxsp;
  rovm_stack_t *minsp;

  /** Local Variable Pointer.  ̰ ڹ  ӽſ   
      Ʒ  `stack' 迭 ϳ slot  Ű ȴ.  ̿  ڼ
       `Stack  Local  '  ϱ ٶ.

      @note  `Stack  Local  '   ʴ´.  */
  rovm_stack_t *local;
  /** Local  ִ slot      , iload  
      opcode  ߸ local     ħϴ Ǵϴµ,
      ȴ.  */
  rovm_stack_t *maxlocal;

  /** Ticket  ϴ  slot .   迭Դϴ. */
  rovm_stack_t *stack;
  /**   `stack'  Ҵ  slot  .  */
  int max_stack_slot;

  /**  ticket operation  óǸ鼭 ȯϰ Ǵ return  Ǵ
      ġ.  */
  rovm_stack_t ret;

  /** GC ޸𸮰 Ҵǰ Ticket Slot    ش ޸ Ͱ
      ӹ ̴.  */
  void **real_delayed_slot;
  int delayed_slot_max;
  int delayed_slot_nelm;
};

extern char *create_ticketid (request_rec *);
extern int gen_unique_ticketid (char *);
extern int register_ticketid (request_rec *, char *);
extern struct ticket *lookup_ticket (request_rec *, char *);
extern int ticket_pre_config ( struct ticket *);
extern int ticket_beforecall (struct ticket *, RvMethod *, u2, u2 *);
extern void destroy_ticket (struct ticket *);
extern int destroy_tickets (request_rec *, char *);
extern void ticket_clear_delay_slot (struct ticket *);

#endif
